#include "RegisterBattleClass.hpp"
#include <CreatureLib/Battling/Models/Battle.hpp>
#include <cstdint>
#include "../../../../Battling/Battle/Battle.hpp"
#include "../HelperFile.hpp"

void RegisterBattleClass::Register(asIScriptEngine* engine) {
    RegisterChoiceQueue(engine);
    RegisterBattleRandom(engine);
    RegisterBattle(engine);
}

void RegisterBattleClass::RegisterChoiceQueue(asIScriptEngine* engine) {
    [[maybe_unused]] int r = engine->RegisterObjectType("ChoiceQueue", 0, asOBJ_REF | asOBJ_NOCOUNT);
    Assert(r >= 0);
    r = engine->RegisterObjectMethod("ChoiceQueue", "bool MovePokemonChoiceNext(Pokemon@ target)",
                                     asMETHOD(CreatureLib::Battling::ChoiceQueue, MoveCreatureChoiceNext),
                                     asCALL_THISCALL);
    Assert(r >= 0);
    r = engine->RegisterObjectMethod("ChoiceQueue", "const BaseTurnChoice@ Peek() const",
                                     asMETHOD(CreatureLib::Battling::ChoiceQueue, Peek), asCALL_THISCALL);
    Assert(r >= 0);
}

void RegisterBattleClass::RegisterBattleRandom(asIScriptEngine* engine) {
    [[maybe_unused]] int r = engine->RegisterObjectType("BattleRandom", 0, asOBJ_REF | asOBJ_NOCOUNT);
    Assert(r >= 0);
    r = engine->RegisterObjectMethod("BattleRandom",
                                     "bool EffectChance(float chance, ExecutingMove@ move, Pokemon@ target )",
                                     asMETHOD(CreatureLib::Battling::BattleRandom, EffectChance), asCALL_THISCALL);
    Assert(r >= 0);
    r = engine->RegisterObjectMethod("BattleRandom", "int Get()",
                                     asMETHODPR(CreatureLib::Battling::BattleRandom, Get, (), int32_t),
                                     asCALL_THISCALL);
    Assert(r >= 0);
    r = engine->RegisterObjectMethod("BattleRandom", "int Get(int max)",
                                     asMETHODPR(CreatureLib::Battling::BattleRandom, Get, (int32_t), int32_t),
                                     asCALL_THISCALL);
    Assert(r >= 0);
    r = engine->RegisterObjectMethod("BattleRandom", "int Get(int min, int max)",
                                     asMETHODPR(CreatureLib::Battling::BattleRandom, Get, (int32_t, int32_t), int32_t),
                                     asCALL_THISCALL);
    Assert(r >= 0);
}

BORROWED_PTR_GETTER_FUNC(PkmnLib::Battling::Battle, CreatureLib::Battling::ChoiceQueue, GetCurrentTurnQueue);

void RegisterBattleClass::RegisterBattle(asIScriptEngine* engine) {
    [[maybe_unused]] int r = engine->RegisterObjectType("Battle", 0, asOBJ_REF | asOBJ_NOCOUNT);
    Assert(r >= 0);
    r = engine->RegisterObjectMethod("Battle", "const BattleLibrary@ get_Library() const property",
                                     asMETHOD(PkmnLib::Battling::Battle, GetLibrary), asCALL_THISCALL);
    Assert(r >= 0);
    r = engine->RegisterObjectMethod("Battle", "bool CanUse(BaseTurnChoice@ choice)",
                                     asMETHOD(PkmnLib::Battling::Battle, CanUse), asCALL_THISCALL);
    Assert(r >= 0);
    r = engine->RegisterObjectMethod("Battle", "bool get_CanFlee() const property",
                                     asMETHOD(PkmnLib::Battling::Battle, CanFlee), asCALL_THISCALL);
    Assert(r >= 0);
    r = engine->RegisterObjectMethod("Battle", "BattleRandom@ get_Random() const property",
                                     asMETHOD(PkmnLib::Battling::Battle, GetRandom), asCALL_THISCALL);
    Assert(r >= 0);
    r = engine->RegisterObjectMethod("Battle", "ChoiceQueue@ get_TurnQueue() const property",
                                     asFUNCTION(GetCurrentTurnQueueWrapper), asCALL_CDECL_OBJFIRST);
    Assert(r >= 0);
    r = engine->RegisterObjectMethod(
        "Battle", "void AddVolatile(const constString &in name) const",
        asMETHODPR(PkmnLib::Battling::Battle, AddVolatileScript, (const ArbUt::StringView&), void), asCALL_THISCALL);
    Assert(r >= 0);
    r = engine->RegisterObjectMethod(
        "Battle", "void RemoveVolatile(const constString &in name) const",
        asMETHODPR(PkmnLib::Battling::Battle, RemoveVolatileScript, (const ArbUt::BasicStringView&), void),
        asCALL_THISCALL);
    Assert(r >= 0);
    r = engine->RegisterObjectMethod(
        "Battle", "void SetWeather(const constString &in name) const",
        asMETHODPR(PkmnLib::Battling::Battle, SetWeather, (const ArbUt::StringView&), void), asCALL_THISCALL);
    Assert(r >= 0);
    r = engine->RegisterObjectMethod("Battle", "void ClearWeather(const constString &in name) const",
                                     asMETHODPR(PkmnLib::Battling::Battle, ClearWeather, (), void), asCALL_THISCALL);
    Assert(r >= 0);
    r = engine->RegisterObjectMethod(
        "Battle", "const constString& GetWeatherName() const",
        asMETHODPR(PkmnLib::Battling::Battle, GetWeatherName, (), const ArbUt::StringView&), asCALL_THISCALL);
    Assert(r >= 0);
}
